/*****************************************************************************
 (c) Copyright 2010-2015 Freescale Semiconductor, Inc.
 ALL RIGHTS RESERVED.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the <organization> nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************

******************************************************************************
*                                                                            *
*        Standard Software Flash Driver For FTFx                             *
*                                                                            *
* FILE NAME     :  PFlashAcessProtection.c                                   *
* DATE          :  Jan 13, 2015                                              *
*                                                                            *
* AUTHOR        :  FPT Team                                                  *
* E-mail        :  r56611@freescale.com                                      *
*                                                                            *
******************************************************************************/

/************************** CHANGES ******************************************
1.0.4       01.13.2015      FPT Team      Initial Version
******************************************************************************/
/* include the header files */

#include "SSD_FTFx.h"

/************************************************************************
*
*  Function Name    : PFlashSetAccessProtection.c
*  Description      : This function sets the P-Flash access protection to the
*                     intended protection status
*  Arguments        : PFLASH_SSD_CONFIG, uint8_t, uint32_t, uint32_t, pFLASHCOMMANDSEQUENCE
*  Return Value     : uint32_t
*
*************************************************************************/
#ifdef RECORD_INDEX_FAC_START
uint32_t PFlashSetAccessProtection(PFLASH_SSD_CONFIG pSSDConfig, \
                                      uint8_t optionProtect,    \
                                      uint32_t expProtectLow, \
                                      uint32_t expProtectHigh, \
                        pFLASHCOMMANDSEQUENCE pFlashCommandSequence)
{
    uint32_t ret = FTFx_OK;             /* return code variable */
    uint32_t protectStatus_1[2];
    uint32_t protectStatus_2[2];
    uint32_t temp1;
    uint32_t temp2;
    uint8_t recordIndex;
    uint8_t recordIndexOffset;
    uint8_t i;

    if ( EXECUTE_ONLY_ACCESS == optionProtect )
    {
        recordIndexOffset = 0x0U;
    }
    else if ( SUPERVISOR_ONLY_ACCESS == optionProtect)
    {
        recordIndexOffset = 0x02U;
    }
    else
    {
        ret = FTFx_ERROR_OPTION_PROTECT;
    }

    if (FTFx_OK == ret)
    {
        recordIndex = recordIndexOffset + RECORD_INDEX_FAC_START;
        ret = FlashReadOnce(pSSDConfig, recordIndex,(uint8_t *)protectStatus_1, pFlashCommandSequence);
        ret |= FlashReadOnce(pSSDConfig, recordIndex + 0x1U,(uint8_t *)protectStatus_2, pFlashCommandSequence);
        protectStatus_1[0] &= protectStatus_2[0];
        protectStatus_1[1] &= protectStatus_2[1];
    }

    if ( FTFx_OK == ret )
    {
        /* Check if possible to make this transition */
        for ( i = 0x0U; i < 32; i++)
        {
            temp1 = (protectStatus_1[0] >> i) & 0x1U;
            temp2 = (expProtectHigh >> i) & 0x1U;
            /* Cannot decrease the range of access protection */
            if ((0x0U == temp1) && (0x1U == temp2))
            {
                ret = FTFx_ERR_CHANGEPROT;
            }
            temp1 = (protectStatus_1[1] >> i) & 0x1U;
            temp2 = (expProtectLow >> i) & 0x1U;
            /* Cannot decrease the range of access protection */
            if ((0x0U == temp1) && (0x1U == temp2))
            {
                ret = FTFx_ERR_CHANGEPROT;
            }
        }
    }

    if(FTFx_OK == ret)
    {
        protectStatus_1[0] &= expProtectHigh;
        protectStatus_1[1] &= expProtectLow;
        ret = FlashProgramOnce(pSSDConfig, recordIndex,(uint8_t *)protectStatus_1, pFlashCommandSequence);
        if(FTFx_ERR_ACCERR == ret)
        {
            ret = FlashProgramOnce(pSSDConfig, recordIndex + 0x1U,(uint8_t *)protectStatus_1, pFlashCommandSequence);
        }
    }

#if C90TFS_ENABLE_DEBUG
    /* Enter Debug state if enabled */
    if (TRUE == (pSSDConfig->DebugEnable))
    {
        ENTER_DEBUG_MODE;
    }
#endif

    return (ret);
}
#endif
/* End of file */
