#ifndef _C90TFS_DEVICE_H_
#define _C90TFS_DEVICE_H_

#include "SSD_Types.h"

/* one block for each device/algorithm */
/*-----------------------------------------------------------------------*/
#if   defined KWx0Z
  #if FLASH_DERIVATIVE != FTFx_KX_160K_0K_0K_1K_0K
    #error "C90TFS_Device does not match user_cfg.h file."
  #endif

  #if   defined P128_1KBSec
    #define FTFx_REG_BASE                 0x40020000u       /* FTFx module base, see reference manual */
    #define PFLASH_BLOCK_BASE             0x00000000u
    #define PFLASH_BLOCK_SIZE             0x00020000u       /* 128 KB size */
    #define DEFLASH_BLOCK_BASE            0x10000000u
    #define DEFLASH_BLOCK_SIZE            0x00000000u
    #define EERAM_BLOCK_BASE              0x14000000u
    #define EERAM_BLOCK_SIZE              0x00000000u
  #elif defined P160_1KBSec
    #define FTFx_REG_BASE                 0x40020000u       /* FTFx module base, see reference manual */
    #define PFLASH_BLOCK_BASE             0x00000000u
    #define PFLASH_BLOCK_SIZE             0x00028000u       /* 160 KB size */
    #define DEFLASH_BLOCK_BASE            0x10000000u
    #define DEFLASH_BLOCK_SIZE            0x00000000u
    #define EERAM_BLOCK_BASE              0x14000000u
    #define EERAM_BLOCK_SIZE              0x00000000u
  #else
    #error "C90TFS_Device does contain Flash sizes."
  #endif

  /* destination to program security key back to flash location */
  #define SECURITY_LOCATION             0x40C               /* because we write 4 Bytes */
  #define SECURITY_SIZE                 PGM_SIZE_BYTE

static __inline void Watchdog_Disable(void) {
  (*((vuint32_t *)(0x40048100u))) = 0x00u;                  /* SIM->COPC = (uint32_t)0x00u; */
}

static __inline void RunMode_Set(void) {
  __nop();
}

#elif defined KWx1Z
  #if FLASH_DERIVATIVE != FTFx_KX_512K_0K_0K_2K_0K
    #error "C90TFS_Device does not match user_cfg.h file."
  #endif

  #if   defined P256_2KBSec
    #define FTFx_REG_BASE                 0x40020000u       /* FTFx module base, see reference manual */
    #define PFLASH_BLOCK_BASE             0x00000000u
    #define PFLASH_BLOCK_SIZE             0x00040000u       /* 256 KB size */
    #define DEFLASH_BLOCK_BASE            0x10000000u
    #define DEFLASH_BLOCK_SIZE            0x00000000u
    #define EERAM_BLOCK_BASE              0x14000000u
    #define EERAM_BLOCK_SIZE              0x00000000u
  #elif defined P512_2KBSec
    #define FTFx_REG_BASE                 0x40020000u       /* FTFx module base, see reference manual */
    #define PFLASH_BLOCK_BASE             0x00000000u
    #define PFLASH_BLOCK_SIZE             0x00080000u       /* 512 KB size */
    #define DEFLASH_BLOCK_BASE            0x10000000u
    #define DEFLASH_BLOCK_SIZE            0x00000000u
    #define EERAM_BLOCK_BASE              0x14000000u
    #define EERAM_BLOCK_SIZE              0x00000000u
  #else
    #error "C90TFS_Device does contain Flash sizes."
  #endif

  /* destination to program security key back to flash location */
  #define SECURITY_LOCATION             0x40C               /* because we write 4 Bytes */
  #define SECURITY_SIZE                 PGM_SIZE_BYTE

static __inline void Watchdog_Disable(void) {
  (*((vuint32_t *)(0x40048100u))) = 0x00u;                  /* SIM->COPC = (uint32_t)0x00u; */
}

static __inline void RunMode_Set(void) {
  __nop();
}

/*-----------------------------------------------------------------------*/
#elif defined KW2xD
  #if FLASH_DERIVATIVE != FTFx_KX_256K_64K_4K_2K_2K
    #error "C90TFS_Device does not match user_cfg.h file."
  #endif

  #if   defined P256_2KBSec || defined D64_2KBSec
    #define FTFx_REG_BASE                 0x40020000u       /* FTFx module base, see reference manual */
    #define PFLASH_BLOCK_BASE             0x00000000u
    #define PFLASH_BLOCK_SIZE             0x00040000u       /* 256 KB size */
    #define DEFLASH_BLOCK_BASE            0x10000000u
    #define DEFLASH_BLOCK_SIZE            0x00010000u       /*  64 KB size */
    #define EERAM_BLOCK_BASE              0x14000000u
    #define EERAM_BLOCK_SIZE              0x00001000u
  #elif defined P512_2KBSec
    #define FTFx_REG_BASE                 0x40020000u       /* FTFx module base, see reference manual */
    #define PFLASH_BLOCK_BASE             0x00000000u
    #define PFLASH_BLOCK_SIZE             0x00080000u       /* 512 KB size */
    #define DEFLASH_BLOCK_BASE            0x10000000u
    #define DEFLASH_BLOCK_SIZE            0x00000000u
    #define EERAM_BLOCK_BASE              0x14000000u
    #define EERAM_BLOCK_SIZE              0x00001000u
  #else
    #error "C90TFS_Device does contain Flash sizes."
  #endif

  /* destination to program security key back to flash location */
  #define SECURITY_LOCATION             0x40C               /* because we write 4 Bytes */
  #define SECURITY_SIZE                 PGM_SIZE_BYTE

static __inline void Watchdog_Disable(void) {
  (*((vuint32_t *)(0x4005200Eu))) = 0xC520u;                /* WDOG->UNLOCK = (uint16_t)0xC520u; */
  (*((vuint32_t *)(0x4005200Eu))) = 0xD928u;                /* WDOG->UNLOCK = (uint16_t)0xD928u; */
  (*((vuint32_t *)(0x40052000u))) = 0x01D2u;                /* WDOG->STCTRLH = (uint16_t)0x01D2u; */
}

static __inline void RunMode_Set(void) {
  __nop();
}



#else
  #error "C90TFS_Device not configured."
#endif

#endif /* #if _C90TFS_DEVICE_H_ */
